<?php

namespace Drupal\regex_text_replacement\Plugin\Filter;

use Drupal\Core\Form\FormStateInterface;
use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;

/**
 * Provides a 'Regex Text Replacement' filter.
 *
 * @Filter(
 *   id = "regex_text_replacement_regex_replacement",
 *   title = @Translation("Regex Text Replacement"),
 *   description = "Apply a Regex filter replacement before rendering content.",
 *   type = Drupal\filter\Plugin\FilterInterface::TYPE_TRANSFORM_REVERSIBLE,
 *   settings = {
 *     "replacements" = "",
 *   },
 *   weight = -10
 * )
 */
class RegexReplacementFilter extends FilterBase {

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $description = <<<HTML
<p>Replace regex matches by any text. Use the format "regex||replacement", one
per line.</p>
<p>Often times you'll more complex Regular Expressions to replace text content,
specially if you are replacing HTML Tags. In order to do that, you can use
parenthesis as placeholders, such as the example bellow.
<p><strong>Ex1: In order to replace all H2 tag per H3 tag.</strong>
<br>/\&lt;h2(.*)\&gt;(.*)&lt;\/h2.*\&gt;/mi||&lt;h3(1)&gt;(2)&lt;/h3&gt;
<br>Note that in both parameters we have two groups of matches (using
parenthesis). The groups from the first parameter (regex) are respectively the
tag attributes, and the content.
<br>The second parameter (replacement), is very similar to the first parameter,
but it is a string instead, that uses the position of each group to properly
fill out both groups: the attributes, and the content.
</p>
<p><strong>Ex2: In order to replace all H1, and H2 tag per H3 tag.</strong>
<br>/\&lt;h[1|2](.*)\&gt(.*)&lt;\/h[1|2].*\&gt/mi||&lt;h3(1)&gt;(2)&lt;/h3&gt;
</p>
HTML;

    $form['replacements'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Replacements'),
      '#description' => $this->t($description),
      '#default_value' => $this->settings['replacements'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function process($text, $langcode) {
    $replacements = $this->settings['replacements'];
    $replacement_items = explode("\n", $replacements);

    // Loop through all replacement items.
    foreach ($replacement_items as $replacement_item) {
      if (!stristr($replacement_item, '||')) {
        continue;
      }

      [$pattern, $replacement_pattern] = explode('||', $replacement_item);

      // Create the correct replacement based on the replacement pattern.
      $replacement = preg_replace_callback('/\((\d*)\)/', function ($matches) {
        return "\${$matches[1]}";
      }, $replacement_pattern);

      // Modify the field value considering the regex values.
      $text = preg_replace_callback($pattern, function ($matches) use ($pattern, $replacement) {
        return preg_replace($pattern, $replacement, $matches[0]);
      }, $text);
    }

    return new FilterProcessResult($text);
  }

  /**
   * {@inheritdoc}
   */
  public function tips($long = FALSE) {
    return $this->t('Apply a Regex filter replacement before rendering the content.');
  }

}
